const fs = require("fs");
const path = require("path");

let packageJsonPath = process.cwd();

while (!fs.existsSync(path.join(packageJsonPath, "package.json"))) {
  packageJsonPath = path.dirname(packageJsonPath);
  if (packageJsonPath === path.resolve("/")) {
    packageJsonPath = null;
    break;
  }
}

let workspacePackageJsonPath = packageJsonPath
  ? path.dirname(packageJsonPath)
  : "/";

while (!fs.existsSync(path.join(workspacePackageJsonPath, "package.json"))) {
  workspacePackageJsonPath = path.dirname(workspacePackageJsonPath);
  if (workspacePackageJsonPath === path.resolve("/")) {
    workspacePackageJsonPath = null;
    break;
  }
}

const packageJson =
  packageJsonPath === null
    ? null
    : require(path.join(packageJsonPath, "package.json"));

let workspaceJson =
  workspacePackageJsonPath === null
    ? null
    : require(path.join(workspacePackageJsonPath, "package.json"));

if (!workspaceJson?.workspaces) {
  workspaceJson = null;
}

const dependencies = new Set([
  ...(packageJson?.dependencies ? Object.keys(packageJson.dependencies) : []),
  ...(packageJson?.devDependencies
    ? Object.keys(packageJson.devDependencies)
    : []),
  ...(workspaceJson?.dependencies
    ? Object.keys(workspaceJson?.dependencies)
    : []),
  ...(workspaceJson?.devDependencies
    ? Object.keys(workspaceJson?.devDependencies)
    : []),
]);

const has = (...libs) => libs.some((name) => dependencies.has(name));

module.exports = {
  react: has("react"),
  next: has("next"),
  typescript: has("typescript"),
  jest: has("jest"),
};
