// this extends eslint:recommended
const defaultRules = {
  "prefer-const": "warn",
  "no-useless-escape": "warn",
  "no-unused-vars": [
    "error",
    {
      ignoreRestSiblings: true,
      args: "all",
      argsIgnorePattern: "^_",
    },
  ],
  "no-shadow": "error",
  "no-empty-function": "off",
  "no-void": "off",
  "promise/no-return-wrap": "error",
  "promise/param-names": "warn",
  // analysis/correctness
  "import/no-unresolved": "error",
  "import/named": "error",
  "import/namespace": "error",
  "import/default": "error",
  "import/export": "error",
  // red flags (thus, warnings)
  "import/no-named-as-default": "off",
  "import/no-named-as-default-member": "off",
  "import/no-duplicates": "off",
  "import/no-extraneous-dependencies": "error",
  "import/extensions": "off",
  "import/prefer-default-export": "off",
  "sonarjs/max-switch-cases": "error",
  "sonarjs/no-all-duplicated-branches": "error",
  "sonarjs/no-collapsible-if": "error",
  "sonarjs/no-collection-size-mischeck": "error",
  "sonarjs/no-duplicated-branches": "error",
  "sonarjs/no-element-overwrite": "error",
  "sonarjs/no-empty-collection": "warn",
  "sonarjs/no-extra-arguments": "warn",
  "sonarjs/no-gratuitous-expressions": "warn",
  "sonarjs/no-identical-conditions": "warn",
  "sonarjs/no-identical-expressions": "warn",
  "sonarjs/no-identical-functions": "warn",
  "sonarjs/no-ignored-return": "warn",
  "sonarjs/no-nested-switch": "error",
  "sonarjs/no-nested-template-literals": "error",
  "sonarjs/no-one-iteration-loop": "error",
  "sonarjs/no-redundant-boolean": "error",
  "sonarjs/no-redundant-jump": "error",
  "sonarjs/no-same-line-conditional": "error",
  "sonarjs/no-small-switch": "error",
  "sonarjs/no-unused-collection": "error",
  "sonarjs/no-use-of-empty-return-value": "error",
  "sonarjs/no-useless-catch": "warn",
  "sonarjs/non-existent-operator": "error",
  "sonarjs/prefer-object-literal": "error",
  "sonarjs/prefer-single-boolean-return": "error",
  "sonarjs/prefer-while": "error",
  "unicorn/consistent-function-scoping": "error",
  "unicorn/empty-brace-spaces": "error",
  "unicorn/error-message": "error",
  "unicorn/escape-case": "error",
  "unicorn/expiring-todo-comments": "error",
  "unicorn/explicit-length-check": "error",
  "unicorn/import-style": "error",
  "unicorn/new-for-builtins": "error",
  "unicorn/no-array-method-this-argument": "error",
  "unicorn/no-array-push-push": "warn",
  "unicorn/no-console-spaces": "error",
  "unicorn/no-document-cookie": "error",
  "unicorn/no-empty-file": "error",
  "unicorn/no-for-loop": "warn",
  "unicorn/no-hex-escape": "error",
  "unicorn/no-instanceof-array": "error",
  "unicorn/no-invalid-remove-event-listener": "error",
  "unicorn/no-nested-ternary": "error",
  "unicorn/no-new-array": "error",
  "unicorn/no-new-buffer": "error",
  "unicorn/no-object-as-default-parameter": "error",
  "unicorn/no-process-exit": "off", // this is app-specific
  "unicorn/no-static-only-class": "error",
  "unicorn/no-thenable": "error",
  "unicorn/no-this-assignment": "error",
  "unicorn/no-unreadable-array-destructuring": "error",
  "unicorn/no-unreadable-iife": "error",
  "unicorn/no-useless-length-check": "warn",
  "unicorn/no-useless-promise-resolve-reject": "warn",
  "unicorn/no-useless-spread": "warn",
  "unicorn/no-useless-switch-case": "warn",
  "unicorn/no-zero-fractions": "error",
  "unicorn/number-literal-case": "error",
  "unicorn/prefer-add-event-listener": "warn",
  "unicorn/prefer-array-find": "warn",
  "unicorn/prefer-array-flat": "warn",
  "unicorn/prefer-array-flat-map": "warn",
  "unicorn/prefer-array-index-of": "warn",
  "unicorn/prefer-array-some": "warn",
  "unicorn/relative-url-style": "error",
  "unicorn/require-array-join-separator": "error",
  "unicorn/require-number-to-fixed-digits-argument": "error",
  "unicorn/template-indent": "error",
  "unicorn/throw-new-error": "error",
};

const nextRules = {
  "react/react-in-jsx-scope": "off",
  "@next/next/no-css-tags": "warn",
  "@next/next/no-sync-scripts": "warn",
  "@next/next/no-html-link-for-pages": "warn",
  "@next/next/no-img-element": "warn",
  "@next/next/no-head-element": "warn",
  "@next/next/no-unwanted-polyfillio": "warn",
  "@next/next/no-page-custom-font": "warn",
  "@next/next/no-title-in-document-head": "warn",
  "@next/next/google-font-display": "warn",
  "@next/next/google-font-preconnect": "warn",
  "@next/next/next-script-for-ga": "warn",
  "@next/next/no-document-import-in-page": "error",
  "@next/next/no-head-import-in-document": "error",
  "@next/next/no-script-component-in-head": "error",
  "@next/next/no-typos": "warn",
  "@next/next/no-duplicate-head": "error",
  "@next/next/inline-script-id": "error",
  "@next/next/no-before-interactive-script-outside-document": "warn",
  "@next/next/no-assign-module-variable": "error",
};
const tsRules = {
  "import/named": "off", // https://github.com/import-js/eslint-plugin-import/blob/main/config/typescript.js#L29
  "constructor-super": "off", // ts(2335) & ts(2377)
  "getter-return": "off", // ts(2378)
  "no-const-assign": "off", // ts(2588)
  "no-dupe-args": "off", // ts(2300)
  "no-dupe-class-members": "off", // ts(2393) & ts(2300)
  "no-dupe-keys": "off", // ts(1117)
  "no-func-assign": "off", // ts(2539)
  "no-import-assign": "off", // ts(2539) & ts(2540)
  "no-new-symbol": "off", // ts(2588)
  "no-obj-calls": "off", // ts(2349)
  "no-redeclare": "off", // ts(2451)
  "no-setter-return": "off", // ts(2408)
  "no-this-before-super": "off", // ts(2376)
  "no-undef": "off", // ts(2304)
  "no-unreachable": "off", // ts(7027)
  "no-unsafe-negation": "off", // ts(2365) & ts(2360) & ts(2358)
  "no-var": "error", // ts transpiles let/const to var, so no need for vars any more
  "prefer-const": "error", // ts provides better types with const
  "prefer-rest-params": "error", // ts provides better types with rest args over arguments
  "prefer-spread": "error", // ts transpiles spread to apply, so no need for manual apply
  "valid-typeof": "off", // ts(2367)
  "@typescript-eslint/adjacent-overload-signatures": "error",
  "@typescript-eslint/ban-ts-comment": "error",
  "@typescript-eslint/ban-types": "error",
  "no-array-constructor": "off",
  "@typescript-eslint/no-array-constructor": "error",
  "no-empty-function": "off",
  "@typescript-eslint/no-empty-function": defaultRules["no-empty-function"],
  "@typescript-eslint/no-empty-interface": "off",
  "@typescript-eslint/no-explicit-any": "off",
  "@typescript-eslint/no-extra-non-null-assertion": "error",
  "no-extra-semi": "off",
  "@typescript-eslint/no-extra-semi": "error",
  "@typescript-eslint/no-inferrable-types": "error",
  "no-loss-of-precision": "off",
  "@typescript-eslint/no-loss-of-precision": "error",
  "@typescript-eslint/no-misused-new": "error",
  "@typescript-eslint/no-namespace": "error",
  "@typescript-eslint/no-non-null-asserted-optional-chain": "error",
  "@typescript-eslint/no-non-null-assertion": "warn",
  "@typescript-eslint/no-this-alias": "error",
  "@typescript-eslint/no-unnecessary-type-constraint": "warn",
  "no-unused-vars": "off",
  "@typescript-eslint/no-unused-vars": defaultRules["no-unused-vars"],
  "no-shadow": "off",
  "@typescript-eslint/no-shadow": defaultRules["no-shadow"],
  "default-case": "off",
  "@typescript-eslint/switch-exhaustiveness-check": "error",
  "@typescript-eslint/prefer-as-const": "error",
  "@typescript-eslint/prefer-namespace-keyword": "error",
  "@typescript-eslint/triple-slash-reference": "error",
  "@typescript-eslint/no-for-in-array": "error",
};
const reactRules = {
  "react/display-name": "error",
  "react/jsx-key": "error",
  "react/jsx-no-comment-textnodes": "error",
  "react/jsx-no-duplicate-props": "error",
  "react/jsx-no-target-blank": "error",
  "react/jsx-no-undef": "error",
  "react/jsx-uses-react": "error",
  "react/jsx-uses-vars": "error",
  "react/no-children-prop": "error",
  "react/no-danger-with-children": "error",
  "react/no-deprecated": "error",
  "react/no-direct-mutation-state": "error",
  "react/no-find-dom-node": "error",
  "react/no-is-mounted": "error",
  "react/no-render-return-value": "error",
  "react/no-string-refs": "error",
  "react/no-unescaped-entities": "error",
  "react/no-unknown-property": "error",
  "react/no-unsafe": "off",
  "react/prop-types": "off",
  "react/react-in-jsx-scope": "error",
  "react/require-render-return": "error",
  "react-hooks/rules-of-hooks": "error",
  "react-hooks/exhaustive-deps": "warn",
};

module.exports = {
  nextRules,
  tsRules,
  reactRules,
  defaultRules,
};
